import typing

from gi.repository import Gdk
from gi.repository import GObject
from gi.repository import win32

T = typing.TypeVar("T")

_lock = ...  # FIXME Constant
_namespace: str = "GdkWin32"
_version: str = "4.0"

def win32_handle_table_lookup(handle: int) -> None: ...

class Win32Display(Gdk.Display):
    """
    :Constructors:

    ::

        Win32Display(**properties)

    Object GdkWin32Display

    Signals from GdkDisplay:
      opened ()
      closed (gboolean)
      seat-added (GdkSeat)
      seat-removed (GdkSeat)
      setting-changed (gchararray)

    Properties from GdkDisplay:
      composited -> gboolean: composited
      rgba -> gboolean: rgba
      shadow-width -> gboolean: shadow-width
      input-shapes -> gboolean: input-shapes
      dmabuf-formats -> GdkDmabufFormats: dmabuf-formats

    Signals from GObject:
      notify (GParam)
    """

    def add_filter(
        self,
        function: typing.Callable[..., Win32MessageFilterReturn],
        *data: typing.Any,
    ) -> None: ...
    def get_egl_display(self) -> None: ...
    @staticmethod
    def get_wgl_version(display: Gdk.Display) -> typing.Tuple[bool, int, int]: ...
    def remove_filter(
        self,
        function: typing.Callable[..., Win32MessageFilterReturn],
        *data: typing.Any,
    ) -> None: ...
    def set_cursor_theme(self, name: typing.Optional[str], size: int) -> None: ...

class Win32DisplayClass(GObject.GPointer): ...
class Win32DisplayManager(Gdk.DisplayManager): ...
class Win32DisplayManagerClass(GObject.GPointer): ...
class Win32Drag(Gdk.Drag): ...
class Win32DragClass(GObject.GPointer): ...
class Win32GLContext(Gdk.GLContext): ...
class Win32GLContextClass(GObject.GPointer): ...

class Win32HCursor(GObject.Object):
    """
    :Constructors:

    ::

        Win32HCursor(**properties)
        new(display:GdkWin32.Win32Display, handle:int, destroyable:bool) -> GdkWin32.Win32HCursor

    Object GdkWin32HCursor

    Properties from GdkWin32HCursor:
      display -> GdkDisplay: display
      handle -> gpointer: handle
      destroyable -> gboolean: destroyable

    Signals from GObject:
      notify (GParam)
    """

    def new(
        display: Win32Display, handle: int, destroyable: bool
    ) -> Win32HCursor: ...  # FIXME Function

class Win32HCursorClass(GObject.GPointer): ...

class Win32MessageFilterReturn:
    CONTINUE: Win32MessageFilterReturn = 0
    REMOVE: Win32MessageFilterReturn = 1
    denominator = ...  # FIXME Constant
    imag = ...  # FIXME Constant
    numerator = ...  # FIXME Constant
    real = ...  # FIXME Constant

    def as_integer_ratio(self, /): ...  # FIXME Function
    def bit_count(self, /): ...  # FIXME Function
    def bit_length(self, /): ...  # FIXME Function
    def conjugate(self, *args, **kwargs): ...  # FIXME Function
    def from_bytes(bytes, byteorder="big", *, signed=False): ...  # FIXME Function
    def is_integer(self, /): ...  # FIXME Function
    def to_bytes(
        self, /, length=1, byteorder="big", *, signed=False
    ): ...  # FIXME Function

class Win32Monitor(Gdk.Monitor):
    """
    :Constructors:

    ::

        Win32Monitor(**properties)

    Object GdkWin32Monitor

    Signals from GdkMonitor:
      invalidate ()

    Properties from GdkMonitor:
      description -> gchararray: description
      display -> GdkDisplay: display
      manufacturer -> gchararray: manufacturer
      model -> gchararray: model
      connector -> gchararray: connector
      scale-factor -> gint: scale-factor
      scale -> gdouble: scale
      geometry -> GdkRectangle: geometry
      width-mm -> gint: width-mm
      height-mm -> gint: height-mm
      refresh-rate -> gint: refresh-rate
      subpixel-layout -> GdkSubpixelLayout: subpixel-layout
      valid -> gboolean: valid

    Signals from GObject:
      notify (GParam)
    """

    @staticmethod
    def get_workarea(monitor: Gdk.Monitor) -> Gdk.Rectangle: ...

class Win32MonitorClass(GObject.GPointer): ...
class Win32Screen(GObject.Object): ...
class Win32ScreenClass(GObject.GPointer): ...

class Win32Surface(Gdk.Surface):
    """
    :Constructors:

    ::

        Win32Surface(**properties)

    Object GdkWin32Surface

    Signals from GdkSurface:
      layout (gint, gint)
      render (CairoRegion) -> gboolean
      event (gpointer) -> gboolean
      enter-monitor (GdkMonitor)
      leave-monitor (GdkMonitor)

    Properties from GdkSurface:
      cursor -> GdkCursor: cursor
      display -> GdkDisplay: display
      frame-clock -> GdkFrameClock: frame-clock
      mapped -> gboolean: mapped
      width -> gint: width
      height -> gint: height
      scale-factor -> gint: scale-factor
      scale -> gdouble: scale

    Signals from GObject:
      notify (GParam)
    """

    def get_handle(self) -> int: ...
    @staticmethod
    def get_impl_hwnd(surface: Gdk.Surface) -> int: ...
    @staticmethod
    def is_win32(surface: Gdk.Surface) -> bool: ...
    def set_urgency_hint(self, urgent: bool) -> None: ...

class Win32SurfaceClass(GObject.GPointer): ...

class _Win32HCursorFake(GObject.GPointer):
    """
    :Constructors:

    ::

        _Win32HCursorFake()
    """

    parent_instance: GObject.Object = ...
    readonly_handle: int = ...



from gi.repository import GObject
from gi.repository import Gst

CODEC_FORMAT: str = ...
CODEC_ID_ANY: int = ...
CODEC_ID_DISABLE: int = ...
RTP_HEADER_EXTENSION_FORMAT: str = ...

def candidate_list_copy(*args, **kwargs): ...
def codec_list_are_equal(*args, **kwargs): ...
def codec_list_copy(*args, **kwargs): ...
def codec_list_from_keyfile(*args, **kwargs): ...
def error_quark(*args, **kwargs): ...
def media_type_to_string(*args, **kwargs): ...
def parse_error(*args, **kwargs): ...
def rtp_header_extension_list_copy(*args, **kwargs): ...
def rtp_header_extension_list_from_keyfile(*args, **kwargs): ...
def utils_get_default_codec_preferences(*args, **kwargs): ...
def utils_get_default_rtp_header_extension_preferences(*args, **kwargs): ...
def utils_set_bitrate(*args, **kwargs): ...
def value_set_candidate_list(*args, **kwargs): ...

class Candidate:
    base_ip = ...
    base_port = ...
    component_id = ...
    foundation = ...
    ip = ...
    password = ...
    port = ...
    priority = ...
    proto = ...
    ttl = ...
    type = ...
    username = ...

    def new(*args, **kwargs): ...
    def new_full(*args, **kwargs): ...

class CandidateList: ...

class Codec:
    channels = ...
    clock_rate = ...
    encoding_name = ...
    feedback_params = ...
    id = ...
    media_type = ...
    minimum_reporting_interval = ...
    optional_params = ...

    def add_feedback_parameter(*args, **kwargs): ...
    def add_optional_parameter(*args, **kwargs): ...
    def are_equal(*args, **kwargs): ...
    def get_feedback_parameter(*args, **kwargs): ...
    def get_optional_parameter(*args, **kwargs): ...
    def new(*args, **kwargs): ...
    def remove_feedback_parameter(*args, **kwargs): ...
    def remove_optional_parameter(*args, **kwargs): ...
    def to_string(*args, **kwargs): ...

class CodecGList:
    def are_equal(*args, **kwargs): ...
    def from_keyfile(*args, **kwargs): ...

class CodecParameter:
    name = ...
    value = ...

    def free(*args, **kwargs): ...

class Conference(Gst.Bin):
    _padding = ...

    def new_participant(*args, **kwargs): ...
    def new_session(self, media_type: MediaType) -> Session: ...
    def do_new_participant(self, *args, **kwargs): ...
    def do_new_session(self, *args, **kwargs): ...

class ElementAddedNotifier:
    parent = ...
    priv = ...

    def add(*args, **kwargs): ...
    def new(*args, **kwargs): ...
    def remove(*args, **kwargs): ...
    def set_default_properties(*args, **kwargs): ...
    def set_properties_from_file(*args, **kwargs): ...
    def set_properties_from_keyfile(*args, **kwargs): ...

class FeedbackParameter:
    extra_params = ...
    subtype = ...
    type = ...

    def free(*args, **kwargs): ...

class Participant:
    _padding = ...
    mutex = ...
    parent = ...
    priv = ...

class Plugin:
    parent = ...
    priv = ...
    type = ...
    unused = ...

    def list_available(*args, **kwargs): ...
    def register_static(*args, **kwargs): ...

class RtpHeaderExtension:
    direction = ...
    id = ...
    uri = ...

    def are_equal(*args, **kwargs): ...
    def new(*args, **kwargs): ...

class RtpHeaderExtensionGList:
    def from_keyfile(*args, **kwargs): ...

class Session:
    _padding = ...
    parent = ...
    priv = ...

    def codecs_need_resend(*args, **kwargs): ...
    def destroy(*args, **kwargs): ...
    def emit_error(*args, **kwargs): ...
    def get_stream_transmitter_type(*args, **kwargs): ...
    def list_transmitters(*args, **kwargs): ...
    def new_stream(*args, **kwargs): ...
    def parse_codecs_changed(*args, **kwargs): ...
    def parse_send_codec_changed(*args, **kwargs): ...
    def parse_telephony_event_started(*args, **kwargs): ...
    def parse_telephony_event_stopped(*args, **kwargs): ...
    def set_allowed_caps(*args, **kwargs): ...
    def set_codec_preferences(*args, **kwargs): ...
    def set_encryption_parameters(*args, **kwargs): ...
    def set_send_codec(*args, **kwargs): ...
    def start_telephony_event(*args, **kwargs): ...
    def stop_telephony_event(*args, **kwargs): ...
    def do_codecs_need_resend(self, *args, **kwargs): ...
    def do_get_stream_transmitter_type(self, *args, **kwargs): ...
    def do_list_transmitters(self, *args, **kwargs): ...
    def do_new_stream(self, *args, **kwargs): ...
    def do_set_allowed_caps(self, *args, **kwargs): ...
    def do_set_codec_preferences(self, *args, **kwargs): ...
    def do_set_encryption_parameters(self, *args, **kwargs): ...
    def do_set_send_codec(self, *args, **kwargs): ...
    def do_start_telephony_event(self, *args, **kwargs): ...
    def do_stop_telephony_event(self, *args, **kwargs): ...

class Stream:
    _padding = ...
    parent = ...
    priv = ...

    def add_id(*args, **kwargs): ...
    def add_remote_candidates(*args, **kwargs): ...
    def destroy(*args, **kwargs): ...
    def emit_error(*args, **kwargs): ...
    def emit_src_pad_added(*args, **kwargs): ...
    def force_remote_candidates(*args, **kwargs): ...
    def iterate_src_pads(*args, **kwargs): ...
    def parse_component_state_changed(*args, **kwargs): ...
    def parse_local_candidates_prepared(*args, **kwargs): ...
    def parse_new_active_candidate_pair(*args, **kwargs): ...
    def parse_new_local_candidate(*args, **kwargs): ...
    def parse_recv_codecs_changed(*args, **kwargs): ...
    def set_decryption_parameters(*args, **kwargs): ...
    def set_remote_codecs(*args, **kwargs): ...
    def set_transmitter(*args, **kwargs): ...
    def set_transmitter_ht(*args, **kwargs): ...
    def do_add_id(self, *args, **kwargs): ...
    def do_add_remote_candidates(self, *args, **kwargs): ...
    def do_force_remote_candidates(self, *args, **kwargs): ...
    def do_set_decryption_parameters(self, *args, **kwargs): ...
    def do_set_remote_codecs(self, *args, **kwargs): ...
    def do_set_transmitter(self, *args, **kwargs): ...

class StreamTransmitter:
    _padding = ...
    parent = ...
    priv = ...

    def add_remote_candidates(*args, **kwargs): ...
    def emit_error(*args, **kwargs): ...
    def force_remote_candidates(*args, **kwargs): ...
    def gather_local_candidates(*args, **kwargs): ...
    def stop(*args, **kwargs): ...
    def do_add_remote_candidates(self, *args, **kwargs): ...
    def do_force_remote_candidates(self, *args, **kwargs): ...
    def do_gather_local_candidates(self, *args, **kwargs): ...
    def do_stop(self, *args, **kwargs): ...

class Transmitter:
    _padding = ...
    construction_error = ...
    parent = ...
    priv = ...

    def emit_error(*args, **kwargs): ...
    def get_stream_transmitter_type(*args, **kwargs): ...
    def list_available(*args, **kwargs): ...
    def new(*args, **kwargs): ...
    def new_stream_transmitter(*args, **kwargs): ...
    def do_get_stream_transmitter_type(self, *args, **kwargs): ...
    def do_new_stream_transmitter(self, *args, **kwargs): ...

class StreamDirection(GObject.GFlags):
    NONE = ...
    SEND = ...
    RECV = ...
    BOTH = ...

class CandidateType(GObject.GEnum):
    HOST = ...
    SRFLX = ...
    PRFLX = ...
    RELAY = ...
    MULTICAST = ...

class ComponentType(GObject.GEnum):
    NONE = ...
    RTP = ...
    RTCP = ...

class DTMFEvent(GObject.GEnum):
    STAR = ...
    POUND = ...
    A = ...
    B = ...
    C = ...
    D = ...

class DTMFMethod(GObject.GEnum):
    RTP_RFC4733 = ...
    SOUND = ...

class Error(GObject.GEnum):
    CONSTRUCTION = ...
    INTERNAL = ...
    INVALID_ARGUMENTS = ...
    NETWORK = ...
    NOT_IMPLEMENTED = ...
    NEGOTIATION_FAILED = ...
    UNKNOWN_CODEC = ...
    NO_CODECS = ...
    NO_CODECS_LEFT = ...
    CONNECTION_FAILED = ...
    DISPOSED = ...
    ALREADY_EXISTS = ...
    quark = ...

class MediaType(GObject.GEnum):
    AUDIO = ...
    VIDEO = ...
    APPLICATION = ...
    LAST = ...
    to_string = ...

class NetworkProtocol(GObject.GEnum):
    UDP = ...
    TCP = ...
    TCP_PASSIVE = ...
    TCP_ACTIVE = ...
    TCP_SO = ...

class StreamState(GObject.GEnum):
    FAILED = ...
    DISCONNECTED = ...
    GATHERING = ...
    CONNECTING = ...
    CONNECTED = ...
    READY = ...



import typing

from gi.repository import gi
from gi.repository import GLib
from gi.repository import GObject
from typing_extensions import Self

T = typing.TypeVar("T")

TYPE_TAG_N_TYPES: int = 22

def invoke_error_quark() -> int: ...
def type_tag_argument_from_hash_pointer(
    storage_type: TypeTag, hash_pointer: None
) -> Argument: ...
def type_tag_hash_pointer_from_argument(
    storage_type: TypeTag, arg: Argument
) -> None: ...
def type_tag_to_string(type: TypeTag) -> str: ...

class ArgInfo(BaseInfo):
    """
    :Constructors:

    ::

        ArgInfo(**properties)
    """

    parent: BaseInfoStack = ...
    padding: list[None] = ...
    def get_closure_index(self) -> typing.Tuple[bool, int]: ...
    def get_destroy_index(self) -> typing.Tuple[bool, int]: ...
    def get_direction(self) -> Direction: ...
    def get_ownership_transfer(self) -> Transfer: ...
    def get_scope(self) -> ScopeType: ...
    def get_type_info(self) -> TypeInfo: ...
    def is_caller_allocates(self) -> bool: ...
    def is_optional(self) -> bool: ...
    def is_return_value(self) -> bool: ...
    def is_skip(self) -> bool: ...
    def load_type_info(self) -> TypeInfo: ...
    def may_be_null(self) -> bool: ...

class Argument(GObject.GPointer):
    v_boolean = ...  # FIXME Constant
    v_double = ...  # FIXME Constant
    v_float = ...  # FIXME Constant
    v_int = ...  # FIXME Constant
    v_int16 = ...  # FIXME Constant
    v_int32 = ...  # FIXME Constant
    v_int64 = ...  # FIXME Constant
    v_int8 = ...  # FIXME Constant
    v_long = ...  # FIXME Constant
    v_pointer = ...  # FIXME Constant
    v_short = ...  # FIXME Constant
    v_size = ...  # FIXME Constant
    v_ssize = ...  # FIXME Constant
    v_string = ...  # FIXME Constant
    v_uint = ...  # FIXME Constant
    v_uint16 = ...  # FIXME Constant
    v_uint32 = ...  # FIXME Constant
    v_uint64 = ...  # FIXME Constant
    v_uint8 = ...  # FIXME Constant
    v_ulong = ...  # FIXME Constant
    v_ushort = ...  # FIXME Constant

class AttributeIter(GObject.GPointer):
    """
    :Constructors:

    ::

        AttributeIter()
    """

    data: None = ...

class BaseInfo(gi.Fundamental):
    """
    :Constructors:

    ::

        BaseInfo(**properties)
    """
    def clear(self) -> None: ...
    def equal(self, info2: BaseInfo) -> bool: ...
    def get_attribute(self, name: str) -> typing.Optional[str]: ...
    def get_container(self) -> BaseInfo: ...
    def get_name(self) -> typing.Optional[str]: ...
    def get_namespace(self) -> str: ...
    def get_typelib(self) -> Typelib: ...
    def is_deprecated(self) -> bool: ...
    def iterate_attributes(self) -> typing.Tuple[bool, AttributeIter, str, str]: ...
    def ref(self) -> BaseInfo: ...
    def unref(self) -> None: ...

class BaseInfoClass(GObject.GPointer): ...

class BaseInfoStack(GObject.GPointer):
    """
    :Constructors:

    ::

        BaseInfoStack()
    """

    parent_instance: GObject.TypeInstance = ...
    dummy0: int = ...
    dummy1: list[None] = ...
    dummy2: list[int] = ...
    dummy3: list[None] = ...

class CallableInfo(BaseInfo):
    """
    :Constructors:

    ::

        CallableInfo(**properties)
    """
    def can_throw_gerror(self) -> bool: ...
    def get_arg(self, n: int) -> ArgInfo: ...
    def get_async_function(self) -> typing.Optional[CallableInfo]: ...
    def get_caller_owns(self) -> Transfer: ...
    def get_finish_function(self) -> typing.Optional[CallableInfo]: ...
    def get_instance_ownership_transfer(self) -> Transfer: ...
    def get_n_args(self) -> int: ...
    def get_return_attribute(self, name: str) -> typing.Optional[str]: ...
    def get_return_type(self) -> TypeInfo: ...
    def get_sync_function(self) -> typing.Optional[CallableInfo]: ...
    def invoke(
        self,
        function: None,
        in_args: typing.Sequence[Argument],
        out_args: typing.Sequence[Argument],
    ) -> typing.Tuple[bool, Argument]: ...
    def is_async(self) -> bool: ...
    def is_method(self) -> bool: ...
    def iterate_return_attributes(
        self,
    ) -> typing.Tuple[bool, AttributeIter, str, str]: ...
    def load_arg(self, n: int) -> ArgInfo: ...
    def load_return_type(self) -> TypeInfo: ...
    def may_return_null(self) -> bool: ...
    def skip_return(self) -> bool: ...

class CallbackInfo(CallableInfo): ...

class ConstantInfo(BaseInfo):
    """
    :Constructors:

    ::

        ConstantInfo(**properties)
    """
    def get_type_info(self) -> TypeInfo: ...

class EnumInfo(RegisteredTypeInfo):
    """
    :Constructors:

    ::

        EnumInfo(**properties)
    """
    def get_error_domain(self) -> typing.Optional[str]: ...
    def get_method(self, n: int) -> FunctionInfo: ...
    def get_n_methods(self) -> int: ...
    def get_n_values(self) -> int: ...
    def get_storage_type(self) -> TypeTag: ...
    def get_value(self, n: int) -> ValueInfo: ...

class FieldInfo(BaseInfo):
    """
    :Constructors:

    ::

        FieldInfo(**properties)
    """
    def get_flags(self) -> FieldInfoFlags: ...
    def get_offset(self) -> int: ...
    def get_size(self) -> int: ...
    def get_type_info(self) -> TypeInfo: ...

class FlagsInfo(EnumInfo): ...

class FunctionInfo(CallableInfo):
    """
    :Constructors:

    ::

        FunctionInfo(**properties)
    """
    def get_flags(self) -> FunctionInfoFlags: ...
    def get_property(self) -> typing.Optional[PropertyInfo]: ...
    def get_symbol(self) -> str: ...
    def get_vfunc(self) -> typing.Optional[VFuncInfo]: ...

class InterfaceInfo(RegisteredTypeInfo):
    """
    :Constructors:

    ::

        InterfaceInfo(**properties)
    """
    def find_method(self, name: str) -> typing.Optional[FunctionInfo]: ...
    def find_signal(self, name: str) -> typing.Optional[SignalInfo]: ...
    def find_vfunc(self, name: str) -> typing.Optional[VFuncInfo]: ...
    def get_constant(self, n: int) -> ConstantInfo: ...
    def get_iface_struct(self) -> typing.Optional[StructInfo]: ...
    def get_method(self, n: int) -> FunctionInfo: ...
    def get_n_constants(self) -> int: ...
    def get_n_methods(self) -> int: ...
    def get_n_prerequisites(self) -> int: ...
    def get_n_properties(self) -> int: ...
    def get_n_signals(self) -> int: ...
    def get_n_vfuncs(self) -> int: ...
    def get_prerequisite(self, n: int) -> BaseInfo: ...
    def get_property(self, n: int) -> PropertyInfo: ...
    def get_signal(self, n: int) -> SignalInfo: ...
    def get_vfunc(self, n: int) -> VFuncInfo: ...

class ObjectInfo(RegisteredTypeInfo):
    """
    :Constructors:

    ::

        ObjectInfo(**properties)
    """
    def find_method(self, name: str) -> typing.Optional[FunctionInfo]: ...
    def find_method_using_interfaces(
        self, name: str
    ) -> typing.Tuple[typing.Optional[FunctionInfo], BaseInfo]: ...
    def find_signal(self, name: str) -> typing.Optional[SignalInfo]: ...
    def find_vfunc(self, name: str) -> typing.Optional[VFuncInfo]: ...
    def find_vfunc_using_interfaces(
        self, name: str
    ) -> typing.Tuple[typing.Optional[VFuncInfo], BaseInfo]: ...
    def get_abstract(self) -> bool: ...
    def get_class_struct(self) -> typing.Optional[StructInfo]: ...
    def get_constant(self, n: int) -> ConstantInfo: ...
    def get_field(self, n: int) -> FieldInfo: ...
    def get_final(self) -> bool: ...
    def get_fundamental(self) -> bool: ...
    def get_get_value_function_name(self) -> typing.Optional[str]: ...
    def get_interface(self, n: int) -> InterfaceInfo: ...
    def get_method(self, n: int) -> FunctionInfo: ...
    def get_n_constants(self) -> int: ...
    def get_n_fields(self) -> int: ...
    def get_n_interfaces(self) -> int: ...
    def get_n_methods(self) -> int: ...
    def get_n_properties(self) -> int: ...
    def get_n_signals(self) -> int: ...
    def get_n_vfuncs(self) -> int: ...
    def get_parent(self) -> typing.Optional[ObjectInfo]: ...
    def get_property(self, n: int) -> PropertyInfo: ...
    def get_ref_function_name(self) -> typing.Optional[str]: ...
    def get_set_value_function_name(self) -> typing.Optional[str]: ...
    def get_signal(self, n: int) -> SignalInfo: ...
    def get_type_init_function_name(self) -> str: ...
    def get_type_name(self) -> str: ...
    def get_unref_function_name(self) -> typing.Optional[str]: ...
    def get_vfunc(self, n: int) -> VFuncInfo: ...

class PropertyInfo(BaseInfo):
    """
    :Constructors:

    ::

        PropertyInfo(**properties)
    """
    def get_flags(self) -> GObject.ParamFlags: ...
    def get_getter(self) -> typing.Optional[FunctionInfo]: ...
    def get_ownership_transfer(self) -> Transfer: ...
    def get_setter(self) -> typing.Optional[FunctionInfo]: ...
    def get_type_info(self) -> TypeInfo: ...

class RegisteredTypeInfo(BaseInfo):
    """
    :Constructors:

    ::

        RegisteredTypeInfo(**properties)
    """
    def get_g_type(self) -> typing.Type[typing.Any]: ...
    def get_type_init_function_name(self) -> typing.Optional[str]: ...
    def get_type_name(self) -> typing.Optional[str]: ...
    def is_boxed(self) -> bool: ...

class Repository(GObject.Object):
    """
    :Constructors:

    ::

        Repository(**properties)
        new() -> GIRepository.Repository

    Object GIRepository

    Signals from GObject:
      notify (GParam)
    """
    @staticmethod
    def dump(input_filename: str, output_filename: str) -> bool: ...
    @staticmethod
    def dup_default() -> Repository: ...
    def enumerate_versions(self, namespace_: str) -> list[str]: ...
    @staticmethod
    def error_quark() -> int: ...
    def find_by_error_domain(self, domain: int) -> typing.Optional[EnumInfo]: ...
    def find_by_gtype(
        self, gtype: typing.Type[typing.Any]
    ) -> typing.Optional[BaseInfo]: ...
    def find_by_name(self, namespace_: str, name: str) -> typing.Optional[BaseInfo]: ...
    def get_c_prefix(self, namespace_: str) -> typing.Optional[str]: ...
    def get_dependencies(self, namespace_: str) -> list[str]: ...
    def get_immediate_dependencies(self, namespace_: str) -> list[str]: ...
    def get_info(self, namespace_: str, idx: int) -> BaseInfo: ...
    def get_library_path(self) -> list[str]: ...
    def get_loaded_namespaces(self) -> list[str]: ...
    def get_n_infos(self, namespace_: str) -> int: ...
    def get_object_gtype_interfaces(
        self, gtype: typing.Type[typing.Any]
    ) -> list[InterfaceInfo]: ...
    @staticmethod
    def get_option_group() -> GLib.OptionGroup: ...
    def get_search_path(self) -> list[str]: ...
    def get_shared_libraries(self, namespace_: str) -> typing.Optional[list[str]]: ...
    def get_typelib_path(self, namespace_: str) -> typing.Optional[str]: ...
    def get_version(self, namespace_: str) -> str: ...
    def is_registered(
        self, namespace_: str, version: typing.Optional[str] = None
    ) -> bool: ...
    def load_typelib(self, typelib: Typelib, flags: RepositoryLoadFlags) -> str: ...
    @classmethod
    def new(cls) -> Repository: ...
    def prepend_library_path(self, directory: str) -> None: ...
    def prepend_search_path(self, directory: str) -> None: ...
    def require(
        self, namespace_: str, version: typing.Optional[str], flags: RepositoryLoadFlags
    ) -> Typelib: ...
    def require_private(
        self,
        typelib_dir: str,
        namespace_: str,
        version: typing.Optional[str],
        flags: RepositoryLoadFlags,
    ) -> Typelib: ...

class RepositoryClass(GObject.GPointer):
    """
    :Constructors:

    ::

        RepositoryClass()
    """

    parent_class: GObject.ObjectClass = ...

class SignalInfo(CallableInfo):
    """
    :Constructors:

    ::

        SignalInfo(**properties)
    """
    def get_class_closure(self) -> typing.Optional[VFuncInfo]: ...
    def get_flags(self) -> GObject.SignalFlags: ...
    def true_stops_emit(self) -> bool: ...

class StructInfo(RegisteredTypeInfo):
    """
    :Constructors:

    ::

        StructInfo(**properties)
    """
    def find_field(self, name: str) -> typing.Optional[FieldInfo]: ...
    def find_method(self, name: str) -> typing.Optional[FunctionInfo]: ...
    def get_alignment(self) -> int: ...
    def get_copy_function_name(self) -> typing.Optional[str]: ...
    def get_field(self, n: int) -> FieldInfo: ...
    def get_free_function_name(self) -> typing.Optional[str]: ...
    def get_method(self, n: int) -> FunctionInfo: ...
    def get_n_fields(self) -> int: ...
    def get_n_methods(self) -> int: ...
    def get_size(self) -> int: ...
    def is_foreign(self) -> bool: ...
    def is_gtype_struct(self) -> bool: ...

class TypeInfo(BaseInfo):
    """
    :Constructors:

    ::

        TypeInfo(**properties)
    """

    parent: BaseInfoStack = ...
    padding: list[None] = ...
    def argument_from_hash_pointer(self, hash_pointer: None) -> Argument: ...
    def get_array_fixed_size(self) -> typing.Tuple[bool, int]: ...
    def get_array_length_index(self) -> typing.Tuple[bool, int]: ...
    def get_array_type(self) -> ArrayType: ...
    def get_interface(self) -> typing.Optional[BaseInfo]: ...
    def get_param_type(self, n: int) -> typing.Optional[TypeInfo]: ...
    def get_storage_type(self) -> TypeTag: ...
    def get_tag(self) -> TypeTag: ...
    def hash_pointer_from_argument(self, arg: Argument) -> None: ...
    def is_pointer(self) -> bool: ...
    def is_zero_terminated(self) -> bool: ...

class Typelib(GObject.GBoxed):
    """
    :Constructors:

    ::

        new_from_bytes(bytes:GLib.Bytes) -> GIRepository.Typelib
    """
    def get_namespace(self) -> str: ...
    @classmethod
    def new_from_bytes(cls, bytes: GLib.Bytes) -> Typelib: ...
    def ref(self) -> Typelib: ...
    def symbol(self, symbol_name: str) -> typing.Tuple[bool, None]: ...
    def unref(self) -> None: ...

class UnionInfo(RegisteredTypeInfo):
    """
    :Constructors:

    ::

        UnionInfo(**properties)
    """
    def find_method(self, name: str) -> typing.Optional[FunctionInfo]: ...
    def get_alignment(self) -> int: ...
    def get_copy_function_name(self) -> typing.Optional[str]: ...
    def get_discriminator(self, n: int) -> typing.Optional[ConstantInfo]: ...
    def get_discriminator_offset(self) -> typing.Tuple[bool, int]: ...
    def get_discriminator_type(self) -> typing.Optional[TypeInfo]: ...
    def get_field(self, n: int) -> FieldInfo: ...
    def get_free_function_name(self) -> typing.Optional[str]: ...
    def get_method(self, n: int) -> FunctionInfo: ...
    def get_n_fields(self) -> int: ...
    def get_n_methods(self) -> int: ...
    def get_size(self) -> int: ...
    def is_discriminated(self) -> bool: ...

class UnresolvedInfo(BaseInfo): ...

class VFuncInfo(CallableInfo):
    """
    :Constructors:

    ::

        VFuncInfo(**properties)
    """
    def get_address(self, implementor_gtype: typing.Type[typing.Any]) -> None: ...
    def get_flags(self) -> VFuncInfoFlags: ...
    def get_invoker(self) -> typing.Optional[FunctionInfo]: ...
    def get_offset(self) -> int: ...
    def get_signal(self) -> typing.Optional[SignalInfo]: ...

class ValueInfo(BaseInfo):
    """
    :Constructors:

    ::

        ValueInfo(**properties)
    """
    def get_value(self) -> int: ...

class FieldInfoFlags(GObject.GFlags):
    INFO_FLAGS_NONE = 0
    IS_READABLE = 1
    IS_WRITABLE = 2

class FunctionInfoFlags(GObject.GFlags):
    INFO_FLAGS_NONE = 0
    IS_ASYNC = 32
    IS_CONSTRUCTOR = 2
    IS_GETTER = 4
    IS_METHOD = 1
    IS_SETTER = 8
    WRAPS_VFUNC = 16

class RepositoryLoadFlags(GObject.GFlags):
    LAZY = 1
    NONE = 0

class VFuncInfoFlags(GObject.GFlags):
    INFO_FLAGS_NONE = 0
    MUST_CHAIN_UP = 1
    MUST_NOT_OVERRIDE = 4
    MUST_OVERRIDE = 2

class ArrayType(GObject.GEnum):
    ARRAY = 1
    BYTE_ARRAY = 3
    C = 0
    PTR_ARRAY = 2

class Direction(GObject.GEnum):
    IN = 0
    INOUT = 2
    OUT = 1

class InvokeError(GObject.GEnum):
    ARGUMENT_MISMATCH = 2
    FAILED = 0
    SYMBOL_NOT_FOUND = 1

class RepositoryError(GObject.GEnum):
    LIBRARY_NOT_FOUND = 3
    NAMESPACE_MISMATCH = 1
    NAMESPACE_VERSION_CONFLICT = 2
    TYPELIB_NOT_FOUND = 0

class ScopeType(GObject.GEnum):
    ASYNC = 2
    CALL = 1
    FOREVER = 4
    INVALID = 0
    NOTIFIED = 3

class Transfer(GObject.GEnum):
    CONTAINER = 1
    EVERYTHING = 2
    NOTHING = 0

class TypeTag(GObject.GEnum):
    ARRAY = 15
    BOOLEAN = 1
    DOUBLE = 11
    ERROR = 20
    FILENAME = 14
    FLOAT = 10
    GHASH = 19
    GLIST = 17
    GSLIST = 18
    GTYPE = 12
    INT16 = 4
    INT32 = 6
    INT64 = 8
    INT8 = 2
    INTERFACE = 16
    UINT16 = 5
    UINT32 = 7
    UINT64 = 9
    UINT8 = 3
    UNICHAR = 21
    UTF8 = 13
    VOID = 0



from typing import Union

from gi.repository import Gio
from gi.repository import GObject

_lock = ...
_namespace: str = ...
_version: str = ...

def get_resource() -> Gio.Resource: ...

class Device(GObject.Object):
    def get_name(self) -> str: ...
    def has_input(self, type: int, code: int) -> bool: ...
    def has_rumble(self) -> bool: ...
    def has_user_mapping(self) -> bool: ...
    def remove_user_mapping(self) -> None: ...
    def rumble(
        self, strong_magnitude: int, weak_magnitude: int, milliseconds: int
    ) -> bool: ...
    def save_user_mapping(self, mapping_string: str) -> None: ...

class Event:
    def get_absolute(self) -> tuple[bool, int, float]: ...
    def get_button(self) -> tuple[bool, int]: ...
    def get_device(self) -> Device: ...
    def get_event_type(self) -> EventType: ...
    def get_hardware_code(self) -> int: ...
    def get_hardware_index(self) -> int: ...
    def get_hardware_type(self) -> int: ...
    def get_hardware_value(self) -> int: ...
    def get_hat(self) -> tuple[bool, int, int]: ...
    def get_time(self) -> int: ...

class Monitor(GObject.Object):
    def iterate(self) -> MonitorIter: ...
    @classmethod
    def new() -> Monitor: ...

class MonitorIter:
    def next(self) -> tuple[bool, Union[Device, None]]: ...

class EventType(GObject.GEnum):
    EVENT_ABSOLUTE = ...
    EVENT_BUTTON_PRESS = ...
    EVENT_BUTTON_RELEASE = ...
    EVENT_HAT = ...
    EVENT_NOTHING = ...



import typing

from gi.repository import Gdk
from gi.repository import GObject
from gi.repository import Pango
from gi.repository import xlib
from typing_extensions import Self

T = typing.TypeVar("T")

_lock = ...  # FIXME Constant
_namespace: str = "GdkX11"
_version: str = "4.0"

def x11_device_get_id(device: X11DeviceXI2) -> int: ...
def x11_device_manager_lookup(
    device_manager: X11DeviceManagerXI2, device_id: int
) -> typing.Optional[X11DeviceXI2]: ...
def x11_free_compound_text(ctext: int) -> None: ...
def x11_free_text_list(list: str) -> None: ...
def x11_get_server_time(surface: X11Surface) -> int: ...
def x11_get_xatom_by_name_for_display(display: X11Display, atom_name: str) -> int: ...
def x11_get_xatom_name_for_display(display: X11Display, xatom: int) -> str: ...
def x11_lookup_xdisplay(xdisplay: xlib.Display) -> X11Display: ...
def x11_set_sm_client_id(sm_client_id: typing.Optional[str] = None) -> None: ...

class X11AppLaunchContext(Gdk.AppLaunchContext):
    """
    :Constructors:

    ::

        X11AppLaunchContext(**properties)

    Object GdkX11AppLaunchContext

    Properties from GdkAppLaunchContext:
      display -> GdkDisplay: display

    Signals from GAppLaunchContext:
      launch-failed (gchararray)
      launch-started (GAppInfo, GVariant)
      launched (GAppInfo, GVariant)

    Signals from GObject:
      notify (GParam)
    """
    class Props(Gdk.AppLaunchContext.Props):
        display: Gdk.Display

    props: Props = ...
    def __init__(self, display: Gdk.Display = ...) -> None: ...

class X11AppLaunchContextClass(GObject.GPointer): ...

class X11DeviceManagerXI2(GObject.Object):
    """
    :Constructors:

    ::

        X11DeviceManagerXI2(**properties)

    Object GdkX11DeviceManagerXI2

    Properties from GdkX11DeviceManagerXI2:
      display -> GdkDisplay: display
      opcode -> gint: opcode
      major -> gint: major
      minor -> gint: minor

    Signals from GObject:
      notify (GParam)
    """
    class Props(GObject.Object.Props):
        display: Gdk.Display
        major: int
        minor: int
        opcode: int

    props: Props = ...
    def __init__(
        self,
        display: Gdk.Display = ...,
        major: int = ...,
        minor: int = ...,
        opcode: int = ...,
    ) -> None: ...

class X11DeviceManagerXI2Class(GObject.GPointer): ...

class X11DeviceXI2(Gdk.Device):
    """
    :Constructors:

    ::

        X11DeviceXI2(**properties)

    Object GdkX11DeviceXI2

    Properties from GdkX11DeviceXI2:
      device-id -> gint: device-id

    Signals from GdkDevice:
      changed ()
      tool-changed (GdkDeviceTool)

    Properties from GdkDevice:
      display -> GdkDisplay: display
      name -> gchararray: name
      source -> GdkInputSource: source
      has-cursor -> gboolean: has-cursor
      n-axes -> guint: n-axes
      vendor-id -> gchararray: vendor-id
      product-id -> gchararray: product-id
      seat -> GdkSeat: seat
      num-touches -> guint: num-touches
      tool -> GdkDeviceTool: tool
      direction -> PangoDirection: direction
      has-bidi-layouts -> gboolean: has-bidi-layouts
      caps-lock-state -> gboolean: caps-lock-state
      num-lock-state -> gboolean: num-lock-state
      scroll-lock-state -> gboolean: scroll-lock-state
      modifier-state -> GdkModifierType: modifier-state
      layout-names -> GStrv: layout-names
      active-layout-index -> gint: active-layout-index

    Signals from GObject:
      notify (GParam)
    """
    class Props(Gdk.Device.Props):
        device_id: int
        active_layout_index: int
        caps_lock_state: bool
        direction: Pango.Direction
        display: Gdk.Display
        has_bidi_layouts: bool
        has_cursor: bool
        layout_names: typing.Optional[list[str]]
        modifier_state: Gdk.ModifierType
        n_axes: int
        name: str
        num_lock_state: bool
        num_touches: int
        product_id: typing.Optional[str]
        scroll_lock_state: bool
        seat: Gdk.Seat
        source: Gdk.InputSource
        tool: typing.Optional[Gdk.DeviceTool]
        vendor_id: typing.Optional[str]

    props: Props = ...
    def __init__(
        self,
        device_id: int = ...,
        display: Gdk.Display = ...,
        has_cursor: bool = ...,
        name: str = ...,
        num_touches: int = ...,
        product_id: str = ...,
        seat: Gdk.Seat = ...,
        source: Gdk.InputSource = ...,
        vendor_id: str = ...,
    ) -> None: ...

class X11DeviceXI2Class(GObject.GPointer): ...

class X11Display(Gdk.Display):
    """
    :Constructors:

    ::

        X11Display(**properties)

    Object GdkX11Display

    Signals from GdkX11Display:
      xevent (gpointer) -> gboolean

    Signals from GdkDisplay:
      opened ()
      closed (gboolean)
      seat-added (GdkSeat)
      seat-removed (GdkSeat)
      setting-changed (gchararray)

    Properties from GdkDisplay:
      composited -> gboolean: composited
      rgba -> gboolean: rgba
      shadow-width -> gboolean: shadow-width
      input-shapes -> gboolean: input-shapes
      dmabuf-formats -> GdkDmabufFormats: dmabuf-formats

    Signals from GObject:
      notify (GParam)
    """
    class Props(Gdk.Display.Props):
        composited: bool
        dmabuf_formats: Gdk.DmabufFormats
        input_shapes: bool
        rgba: bool
        shadow_width: bool

    props: Props = ...
    def error_trap_pop(self) -> int: ...
    def error_trap_pop_ignored(self) -> None: ...
    def error_trap_push(self) -> None: ...
    def get_default_group(self) -> Gdk.Surface: ...
    def get_egl_display(self) -> None: ...
    def get_egl_version(self) -> typing.Tuple[bool, int, int]: ...
    def get_glx_version(self) -> typing.Tuple[bool, int, int]: ...
    def get_primary_monitor(self) -> Gdk.Monitor: ...
    def get_screen(self) -> X11Screen: ...
    def get_startup_notification_id(self) -> str: ...
    def get_user_time(self) -> int: ...
    def get_xcursor(self, cursor: Gdk.Cursor) -> int: ...
    def get_xdisplay(self) -> xlib.Display: ...
    def get_xrootwindow(self) -> int: ...
    def get_xscreen(self) -> xlib.Screen: ...
    def grab(self) -> None: ...
    @staticmethod
    def open(
        display_name: typing.Optional[str] = None,
    ) -> typing.Optional[Gdk.Display]: ...
    def set_cursor_theme(self, theme: typing.Optional[str], size: int) -> None: ...
    @staticmethod
    def set_program_class(display: Gdk.Display, program_class: str) -> None: ...
    def set_startup_notification_id(self, startup_id: str) -> None: ...
    def set_surface_scale(self, scale: int) -> None: ...
    def string_to_compound_text(
        self, str: str
    ) -> typing.Tuple[int, str, int, bytes]: ...
    def text_property_to_text_list(
        self, encoding: str, format: int, text: int, length: int, list: str
    ) -> int: ...
    def ungrab(self) -> None: ...
    def utf8_to_compound_text(
        self, str: str
    ) -> typing.Tuple[bool, str, int, bytes]: ...

class X11DisplayClass(GObject.GPointer): ...

class X11Drag(Gdk.Drag):
    """
    :Constructors:

    ::

        X11Drag(**properties)

    Object GdkX11Drag

    Signals from GdkDrag:
      cancel (GdkDragCancelReason)
      drop-performed ()
      dnd-finished ()

    Properties from GdkDrag:
      content -> GdkContentProvider: content
      device -> GdkDevice: device
      display -> GdkDisplay: display
      formats -> GdkContentFormats: formats
      selected-action -> GdkDragAction: selected-action
      actions -> GdkDragAction: actions
      surface -> GdkSurface: surface

    Signals from GObject:
      notify (GParam)
    """
    class Props(Gdk.Drag.Props):
        actions: Gdk.DragAction
        content: Gdk.ContentProvider
        device: Gdk.Device
        display: Gdk.Display
        formats: Gdk.ContentFormats
        selected_action: Gdk.DragAction
        surface: Gdk.Surface

    props: Props = ...
    def __init__(
        self,
        actions: Gdk.DragAction = ...,
        content: Gdk.ContentProvider = ...,
        device: Gdk.Device = ...,
        formats: Gdk.ContentFormats = ...,
        selected_action: Gdk.DragAction = ...,
        surface: Gdk.Surface = ...,
    ) -> None: ...

class X11DragClass(GObject.GPointer): ...

class X11GLContext(Gdk.GLContext):
    """
    :Constructors:

    ::

        X11GLContext(**properties)

    Object GdkX11GLContext

    Properties from GdkGLContext:
      allowed-apis -> GdkGLAPI: allowed-apis
      api -> GdkGLAPI: api
      shared-context -> GdkGLContext: shared-context

    Properties from GdkDrawContext:
      display -> GdkDisplay: display
      surface -> GdkSurface: surface

    Signals from GObject:
      notify (GParam)
    """
    class Props(Gdk.GLContext.Props):
        allowed_apis: Gdk.GLAPI
        api: Gdk.GLAPI
        shared_context: typing.Optional[Gdk.GLContext]
        display: typing.Optional[Gdk.Display]
        surface: typing.Optional[Gdk.Surface]

    props: Props = ...
    def __init__(
        self,
        allowed_apis: Gdk.GLAPI = ...,
        shared_context: Gdk.GLContext = ...,
        display: Gdk.Display = ...,
        surface: Gdk.Surface = ...,
    ) -> None: ...

class X11GLContextClass(GObject.GPointer): ...

class X11Monitor(Gdk.Monitor):
    """
    :Constructors:

    ::

        X11Monitor(**properties)

    Object GdkX11Monitor

    Signals from GdkMonitor:
      invalidate ()

    Properties from GdkMonitor:
      description -> gchararray: description
      display -> GdkDisplay: display
      manufacturer -> gchararray: manufacturer
      model -> gchararray: model
      connector -> gchararray: connector
      scale-factor -> gint: scale-factor
      scale -> gdouble: scale
      geometry -> GdkRectangle: geometry
      width-mm -> gint: width-mm
      height-mm -> gint: height-mm
      refresh-rate -> gint: refresh-rate
      subpixel-layout -> GdkSubpixelLayout: subpixel-layout
      valid -> gboolean: valid

    Signals from GObject:
      notify (GParam)
    """
    class Props(Gdk.Monitor.Props):
        connector: typing.Optional[str]
        description: typing.Optional[str]
        display: Gdk.Display
        geometry: Gdk.Rectangle
        height_mm: int
        manufacturer: typing.Optional[str]
        model: typing.Optional[str]
        refresh_rate: int
        scale: float
        scale_factor: int
        subpixel_layout: Gdk.SubpixelLayout
        valid: bool
        width_mm: int

    props: Props = ...
    def __init__(self, display: Gdk.Display = ...) -> None: ...
    def get_output(self) -> int: ...
    def get_workarea(self) -> Gdk.Rectangle: ...

class X11MonitorClass(GObject.GPointer): ...

class X11Screen(GObject.Object):
    """
    :Constructors:

    ::

        X11Screen(**properties)

    Object GdkX11Screen

    Signals from GdkX11Screen:
      window-manager-changed ()

    Signals from GObject:
      notify (GParam)
    """
    def get_current_desktop(self) -> int: ...
    def get_monitor_output(self, monitor_num: int) -> int: ...
    def get_number_of_desktops(self) -> int: ...
    def get_screen_number(self) -> int: ...
    def get_window_manager_name(self) -> str: ...
    def get_xscreen(self) -> xlib.Screen: ...
    def supports_net_wm_hint(self, property_name: str) -> bool: ...

class X11ScreenClass(GObject.GPointer): ...

class X11Surface(Gdk.Surface):
    """
    :Constructors:

    ::

        X11Surface(**properties)

    Object GdkX11Surface

    Signals from GdkSurface:
      layout (gint, gint)
      render (CairoRegion) -> gboolean
      event (gpointer) -> gboolean
      enter-monitor (GdkMonitor)
      leave-monitor (GdkMonitor)

    Properties from GdkSurface:
      cursor -> GdkCursor: cursor
      display -> GdkDisplay: display
      frame-clock -> GdkFrameClock: frame-clock
      mapped -> gboolean: mapped
      width -> gint: width
      height -> gint: height
      scale-factor -> gint: scale-factor
      scale -> gdouble: scale

    Signals from GObject:
      notify (GParam)
    """
    class Props(Gdk.Surface.Props):
        cursor: typing.Optional[Gdk.Cursor]
        display: Gdk.Display
        frame_clock: Gdk.FrameClock
        height: int
        mapped: bool
        scale: float
        scale_factor: int
        width: int

    props: Props = ...
    def __init__(
        self,
        cursor: typing.Optional[Gdk.Cursor] = ...,
        display: Gdk.Display = ...,
        frame_clock: Gdk.FrameClock = ...,
    ) -> None: ...
    def get_desktop(self) -> int: ...
    def get_group(self) -> typing.Optional[Gdk.Surface]: ...
    def get_xid(self) -> int: ...
    @staticmethod
    def lookup_for_display(display: X11Display, window: int) -> X11Surface: ...
    def move_to_current_desktop(self) -> None: ...
    def move_to_desktop(self, desktop: int) -> None: ...
    def set_frame_sync_enabled(self, frame_sync_enabled: bool) -> None: ...
    def set_group(self, leader: Gdk.Surface) -> None: ...
    def set_skip_pager_hint(self, skips_pager: bool) -> None: ...
    def set_skip_taskbar_hint(self, skips_taskbar: bool) -> None: ...
    def set_theme_variant(self, variant: str) -> None: ...
    def set_urgency_hint(self, urgent: bool) -> None: ...
    def set_user_time(self, timestamp: int) -> None: ...
    def set_utf8_property(
        self, name: str, value: typing.Optional[str] = None
    ) -> None: ...

class X11SurfaceClass(GObject.GPointer): ...

class X11DeviceType(GObject.GEnum):
    FLOATING = 2
    LOGICAL = 0
    PHYSICAL = 1
